@@ -280,7 +280,6 @@ gdk_event_type_get_type (void)
             { GDK_DROP_FINISHED, "GDK_DROP_FINISHED", "drop-finished" },
             { GDK_CLIENT_EVENT, "GDK_CLIENT_EVENT", "client-event" },
             { GDK_VISIBILITY_NOTIFY, "GDK_VISIBILITY_NOTIFY", "visibility-notify" },
-            { GDK_NO_EXPOSE, "GDK_NO_EXPOSE", "no-expose" },
             { GDK_SCROLL, "GDK_SCROLL", "scroll" },
             { GDK_WINDOW_STATE, "GDK_WINDOW_STATE", "window-state" },
             { GDK_SETTING, "GDK_SETTING", "setting" },
@@ -606,7 +605,7 @@ gdk_visual_type_get_type (void)
 
 /* enumerations from "gdkwindow.h" */
 GType
-gdk_window_class_get_type (void)
+gdk_window_window_class_get_type (void)
 {
     static GType etype = 0;
     if (G_UNLIKELY(etype == 0)) {
@@ -615,7 +614,7 @@ gdk_window_class_get_type (void)
             { GDK_INPUT_ONLY, "GDK_INPUT_ONLY", "only" },
             { 0, NULL, NULL }
         };
-        etype = g_enum_register_static (g_intern_static_string ("GdkWindowClass"), values);
+        etype = g_enum_register_static (g_intern_static_string ("GdkWindowWindowClass"), values);
     }
     return etype;
 }
