@@ -25,100 +25,12 @@
  */
 
 #include "config.h"
-#include "gdkx.h"
+
 #include "gdkprivate-x11.h"
 #include "gdkdisplay-x11.h"
+
 #include <stdio.h>
 
-static guint     gdk_xid_hash  (XID *xid);
-static gboolean  gdk_xid_equal (XID *a,
-				XID *b);
-
-
-void
-_gdk_xid_table_insert (GdkDisplay *display,
-		       XID	  *xid,
-		       gpointer    data)
-{
-  GdkDisplayX11 *display_x11;
-
-  g_return_if_fail (xid != NULL);
-  g_return_if_fail (GDK_IS_DISPLAY (display));
-
-  display_x11 = GDK_DISPLAY_X11 (display);
-
-  if (!display_x11->xid_ht)
-    display_x11->xid_ht = g_hash_table_new ((GHashFunc) gdk_xid_hash,
-					    (GEqualFunc) gdk_xid_equal);
-
-  if (g_hash_table_lookup (display_x11->xid_ht, xid))
-    g_warning ("XID collision, trouble ahead");
-
-  g_hash_table_insert (display_x11->xid_ht, xid, data);
-}
-
-void
-_gdk_xid_table_remove (GdkDisplay *display,
-		       XID	   xid)
-{
-  GdkDisplayX11 *display_x11;
-
-  g_return_if_fail (GDK_IS_DISPLAY (display));
-
-  display_x11 = GDK_DISPLAY_X11 (display);
-
-  if (display_x11->xid_ht)
-    g_hash_table_remove (display_x11->xid_ht, &xid);
-}
-
-/**
- * gdk_xid_table_lookup_for_display:
- * @display: the #GdkDisplay.
- * @xid: an X id.
- *
- * Returns the GDK object associated with the given X id.
- *
- * Return value: the associated Gdk object, which may be a #GdkPixmap,
- *     a #GdkWindow or %NULL if no object is associated
- *     with the X id.
- *
- * Since: 2.2
- */
-gpointer
-gdk_xid_table_lookup_for_display (GdkDisplay  *display,
-				  XID	       xid)
-{
-  GdkDisplayX11 *display_x11;
-  gpointer data = NULL;
-  
-  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-  
-  display_x11 = GDK_DISPLAY_X11 (display);
-
-  if (display_x11->xid_ht)
-    data = g_hash_table_lookup (display_x11->xid_ht, &xid);
-  
-  return data;
-}
-
-
-/**
- * gdk_xid_table_lookup:
- * @xid: an X id.
- *
- * Returns the Gdk object associated with the given X id for the default
- * display.
- *
- * Return value: the associated Gdk object, which may be a #GdkPixmap,
- *     a #GdkWindow or %NULL if no object is associated
- *     with the X id.
- */
-gpointer
-gdk_xid_table_lookup (XID xid)
-{
-  return gdk_xid_table_lookup_for_display (gdk_display_get_default (), xid);
-}
-
 static guint
 gdk_xid_hash (XID *xid)
 {
@@ -126,8 +38,72 @@ gdk_xid_hash (XID *xid)
 }
 
 static gboolean
-gdk_xid_equal (XID *a,
-	       XID *b)
+gdk_xid_equal (XID *a, XID *b)
 {
   return (*a == *b);
 }
+
+void
+_gdk_x11_display_add_window (GdkDisplay *display,
+                             XID        *xid,
+                             GdkWindow  *data)
+{
+  GdkX11Display *display_x11;
+
+  g_return_if_fail (xid != NULL);
+  g_return_if_fail (GDK_IS_DISPLAY (display));
+
+  display_x11 = GDK_X11_DISPLAY (display);
+
+  if (!display_x11->xid_ht)
+    display_x11->xid_ht = g_hash_table_new ((GHashFunc) gdk_xid_hash,
+                                            (GEqualFunc) gdk_xid_equal);
+
+  if (g_hash_table_lookup (display_x11->xid_ht, xid))
+    g_warning ("XID collision, trouble ahead");
+
+  g_hash_table_insert (display_x11->xid_ht, xid, data);
+}
+
+void
+_gdk_x11_display_remove_window (GdkDisplay *display,
+                                XID         xid)
+{
+  GdkX11Display *display_x11;
+
+  g_return_if_fail (GDK_IS_DISPLAY (display));
+
+  display_x11 = GDK_X11_DISPLAY (display);
+
+  if (display_x11->xid_ht)
+    g_hash_table_remove (display_x11->xid_ht, &xid);
+}
+
+/**
+ * gdk_x11_window_lookup_for_display:
+ * @display: the #GdkDisplay corresponding to the window handle
+ * @window: an XLib <type>Window</type>
+ *
+ * Looks up the #GdkWindow that wraps the given native window handle.
+ *
+ * Return value: (transfer none): the #GdkWindow wrapper for the native
+ *    window, or %NULL if there is none.
+ *
+ * Since: 2.24
+ */
+GdkWindow *
+gdk_x11_window_lookup_for_display (GdkDisplay *display,
+                                   Window      window)
+{
+  GdkX11Display *display_x11;
+  GdkWindow *data = NULL;
+
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
+
+  display_x11 = GDK_X11_DISPLAY (display);
+
+  if (display_x11->xid_ht)
+    data = g_hash_table_lookup (display_x11->xid_ht, &window);
+
+  return data;
+}
