@@ -30,36 +30,33 @@
  * @Title: GtkButtonBox
  * @See_also: #GtkVButtonBox, #GtkHButtonBox
  *
- * The primary purpose of this class is to keep track of the various properties
- * of #GtkHButtonBox and #GtkVButtonBox widgets.
- *
- * gtk_button_box_get_child_size() retrieves the minimum width and height
- * for widgets in a given button box.
- *
- * The internal padding of buttons can be retrieved and changed per button box
- * using gtk_button_box_get_child_ipadding() and
- * gtk_button_box_set_child_ipadding() respectively.
- *
- * gtk_button_box_get_spacing() and gtk_button_box_set_spacing() retrieve and
- * change default number of pixels between buttons, respectively.
+ * The primary purpose of this class is to keep track of the various
+ * properties of #GtkHButtonBox and #GtkVButtonBox widgets.
  *
  * gtk_button_box_get_layout() and gtk_button_box_set_layout() retrieve and
  * alter the method used to spread the buttons in a button box across the
  * container, respectively.
  *
  * The main purpose of GtkButtonBox is to make sure the children have all the
- * same size. Therefore it ignores the homogeneous property which it inherited
- * from GtkBox, and always behaves as if homogeneous was %TRUE.
+ * same size. GtkButtonBox gives all children the same size, but it does allow
+ * 'outliers' to keep their own larger size. To force all children to be
+ * strictly the same size without exceptions, you can set the
+ * #GtkButtonBox::homogeneous property to %TRUE.
  */
 
 #include "config.h"
+
 #include "gtkbbox.h"
+
 #include "gtkorientable.h"
+#include "gtktypebuiltins.h"
 #include "gtkprivate.h"
+#include "gtksizerequest.h"
+
 #include "gtkintl.h"
 
 
-struct _GtkButtonBoxPriv
+struct _GtkButtonBoxPrivate
 {
   GtkButtonBoxStyle layout_style;
 };
@@ -84,8 +81,21 @@ static void gtk_button_box_get_property       (GObject           *object,
                                                guint              prop_id,
                                                GValue            *value,
                                                GParamSpec        *pspec);
-static void gtk_button_box_size_request       (GtkWidget         *widget,
-                                               GtkRequisition    *requisition);
+static void gtk_button_box_get_preferred_width            (GtkWidget *widget,
+                                                           gint      *minimum,
+                                                           gint      *natural);
+static void gtk_button_box_get_preferred_height           (GtkWidget *widget,
+                                                           gint      *minimum,
+                                                           gint      *natural);
+static void gtk_button_box_get_preferred_width_for_height (GtkWidget *widget,
+                                                           gint       height,
+                                                           gint      *minimum,
+                                                           gint      *natural);
+static void gtk_button_box_get_preferred_height_for_width (GtkWidget *widget,
+                                                           gint       width,
+                                                           gint      *minimum,
+                                                           gint      *natural);
+
 static void gtk_button_box_size_allocate      (GtkWidget         *widget,
                                                GtkAllocation     *allocation);
 static void gtk_button_box_remove             (GtkContainer      *container,
@@ -123,12 +133,16 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
   gobject_class->set_property = gtk_button_box_set_property;
   gobject_class->get_property = gtk_button_box_get_property;
 
-  widget_class->size_request = gtk_button_box_size_request;
+  widget_class->get_preferred_width = gtk_button_box_get_preferred_width;
+  widget_class->get_preferred_height = gtk_button_box_get_preferred_height;
+  widget_class->get_preferred_width_for_height = gtk_button_box_get_preferred_width_for_height;
+  widget_class->get_preferred_height_for_width = gtk_button_box_get_preferred_height_for_width;
   widget_class->size_allocate = gtk_button_box_size_allocate;
 
   container_class->remove = gtk_button_box_remove;
   container_class->set_child_property = gtk_button_box_set_child_property;
   container_class->get_child_property = gtk_button_box_get_child_property;
+  gtk_container_class_handle_border_width (container_class);
 
   /* FIXME we need to override the "spacing" property on GtkBox once
    * libgobject allows that.
@@ -172,7 +186,7 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
                                    PROP_LAYOUT_STYLE,
                                    g_param_spec_enum ("layout-style",
                                                       P_("Layout style"),
-                                                      P_("How to layout the buttons in the box. Possible values are spread, edge, start and end"),
+                                                      P_("How to lay out the buttons in the box. Possible values are: spread, edge, start and end"),
                                                       GTK_TYPE_BUTTON_BOX_STYLE,
                                                       DEFAULT_LAYOUT_STYLE,
                                                       GTK_PARAM_READWRITE));
@@ -185,17 +199,17 @@ gtk_button_box_class_init (GtkButtonBoxClass *class)
                                                                     FALSE,
                                                                     GTK_PARAM_READWRITE));
 
-  g_type_class_add_private (class, sizeof (GtkButtonBoxPriv));
+  g_type_class_add_private (class, sizeof (GtkButtonBoxPrivate));
 }
 
 static void
 gtk_button_box_init (GtkButtonBox *button_box)
 {
-  GtkButtonBoxPriv *priv;
+  GtkButtonBoxPrivate *priv;
 
   button_box->priv = G_TYPE_INSTANCE_GET_PRIVATE (button_box,
                                                   GTK_TYPE_BUTTON_BOX,
-                                                  GtkButtonBoxPriv);
+                                                  GtkButtonBoxPrivate);
   priv = button_box->priv;
 
   gtk_box_set_spacing (GTK_BOX (button_box), 0);
@@ -226,7 +240,7 @@ gtk_button_box_get_property (GObject    *object,
                              GValue     *value,
                              GParamSpec *pspec)
 {
-  GtkButtonBoxPriv *priv = GTK_BUTTON_BOX (object)->priv;
+  GtkButtonBoxPrivate *priv = GTK_BUTTON_BOX (object)->priv;
 
   switch (prop_id)
     {
@@ -303,7 +317,7 @@ void
 gtk_button_box_set_layout (GtkButtonBox      *widget,
                            GtkButtonBoxStyle  layout_style)
 {
-  GtkButtonBoxPriv *priv;
+  GtkButtonBoxPrivate *priv;
 
   g_return_if_fail (GTK_IS_BUTTON_BOX (widget));
 
@@ -323,7 +337,7 @@ gtk_button_box_set_layout (GtkButtonBox      *widget,
  *
  * Retrieves the method being used to arrange the buttons in a button box.
  *
- * Returns: the method used to layout buttons in @widget.
+ * Returns: the method used to lay out buttons in @widget.
  */
 GtkButtonBoxStyle
 gtk_button_box_get_layout (GtkButtonBox *widget)
@@ -381,7 +395,7 @@ gtk_button_box_set_child_secondary (GtkButtonBox *widget,
 {
   g_return_if_fail (GTK_IS_BUTTON_BOX (widget));
   g_return_if_fail (GTK_IS_WIDGET (child));
-  g_return_if_fail (child->parent == GTK_WIDGET (widget));
+  g_return_if_fail (gtk_widget_get_parent (child) == GTK_WIDGET (widget));
 
   g_object_set_data (G_OBJECT (child),
                      GTK_BOX_SECONDARY_CHILD,
@@ -394,22 +408,24 @@ gtk_button_box_set_child_secondary (GtkButtonBox *widget,
 }
 
 /* Ask children how much space they require and round up
-   to match minimum size and internal padding.
-   Returns the size each single child should have. */
+ * to match minimum size and internal padding.
+ * Returns the size each single child should have.
+ */
 static void
-_gtk_button_box_child_requisition (GtkWidget *widget,
-                                   int       *nvis_children,
-                                   int       *nvis_secondaries,
-                                   int       *width,
-                                   int       *height)
+gtk_button_box_child_requisition (GtkWidget  *widget,
+                                  gint       *nvis_children,
+                                  gint       *nvis_secondaries,
+                                  gint      **widths,
+                                  gint      **heights)
 {
-  GtkButtonBoxPriv *priv;
+  GtkButtonBoxPrivate *priv;
   GtkButtonBox *bbox;
   GList *children, *list;
   gint nchildren;
   gint nsecondaries;
   gint needed_width;
   gint needed_height;
+  gint avg_w, avg_h;
   GtkRequisition child_requisition;
   gint ipad_w;
   gint ipad_h;
@@ -417,12 +433,16 @@ _gtk_button_box_child_requisition (GtkWidget *widget,
   gint child_min_height;
   gint ipad_x;
   gint ipad_y;
+  gboolean homogeneous;
+  gint i;
 
   g_return_if_fail (GTK_IS_BUTTON_BOX (widget));
 
   bbox = GTK_BUTTON_BOX (widget);
   priv = bbox->priv;
 
+  homogeneous = gtk_box_get_homogeneous (GTK_BOX (widget));
+
   gtk_widget_style_get (widget,
                         "child-min-width", &child_min_width,
                         "child-min-height", &child_min_height,
@@ -437,7 +457,32 @@ _gtk_button_box_child_requisition (GtkWidget *widget,
   needed_height = child_min_height;
   ipad_w = ipad_x * 2;
   ipad_h = ipad_y * 2;
-  
+
+  avg_w = avg_h = 0;
+  while (children)
+    {
+      GtkWidget *child;
+
+      child = children->data;
+      children = children->next;
+
+      if (gtk_widget_get_visible (child))
+        {
+          nchildren += 1;
+          gtk_widget_get_preferred_size (child,
+                                         &child_requisition, NULL);
+          avg_w += child_requisition.width + ipad_w;
+          avg_h += child_requisition.height + ipad_h;
+        }
+    }
+  avg_w /= MAX (nchildren, 1);
+  avg_h /= MAX (nchildren, 1);
+
+  *widths = g_new (gint, nchildren);
+  *heights = g_new (gint, nchildren);
+
+  i = 0;
+  children = list;
   while (children)
     {
       GtkWidget *child;
@@ -446,60 +491,100 @@ _gtk_button_box_child_requisition (GtkWidget *widget,
       child = children->data;
       children = children->next;
 
-      is_secondary = gtk_button_box_get_child_secondary (bbox, child);
-
       if (gtk_widget_get_visible (child))
         {
-          nchildren += 1;
-          gtk_widget_size_request (child, &child_requisition);
-
-          if (child_requisition.width + ipad_w > needed_width)
-            needed_width = child_requisition.width + ipad_w;
-          if (child_requisition.height + ipad_h > needed_height)
-            needed_height = child_requisition.height + ipad_h;
+          is_secondary = gtk_button_box_get_child_secondary (bbox, child);
           if (is_secondary)
             nsecondaries++;
+
+          gtk_widget_get_preferred_size (child, &child_requisition, NULL);
+
+          if (homogeneous || (child_requisition.width + ipad_w < avg_w * 1.5))
+            {
+              (*widths)[i] = -1;
+              if (child_requisition.width + ipad_w > needed_width)
+                needed_width = child_requisition.width + ipad_w;
+            }
+          else
+            {
+              (*widths)[i] = child_requisition.width + ipad_w;
+            }
+
+          if (homogeneous || (child_requisition.height + ipad_h < avg_h * 1.5))
+            {
+              (*heights)[i] = -1;
+              if (child_requisition.height + ipad_h > needed_height)
+                needed_height = child_requisition.height + ipad_h;
+            }
+          else
+            {
+              (*heights)[i] = child_requisition.height + ipad_h;
+            }
+
+          i++;
         }
     }
 
   g_list_free (list);
 
+  for (i = 0; i < nchildren; i++)
+    {
+      if ((*widths)[i] == -1)
+        (*widths)[i] = needed_width;
+      if ((*heights)[i] == -1)
+        (*heights)[i] = needed_height;
+    }
+
   if (nvis_children)
     *nvis_children = nchildren;
+
   if (nvis_secondaries)
     *nvis_secondaries = nsecondaries;
-  if (width)
-    *width = needed_width;
-  if (height)
-    *height = needed_height;
 }
 
 static void
 gtk_button_box_size_request (GtkWidget      *widget,
                              GtkRequisition *requisition)
 {
-  GtkButtonBoxPriv *priv;
-  GtkBox *box;
+  GtkButtonBoxPrivate *priv;
   GtkButtonBox *bbox;
   gint nvis_children;
-  gint child_width;
-  gint child_height;
+  gint max_size;
+  gint total_size;
   gint spacing;
-  guint border_width;
   GtkOrientation orientation;
+  gint *widths;
+  gint *heights;
+  gint i;
 
-  box = GTK_BOX (widget);
   bbox = GTK_BUTTON_BOX (widget);
   priv = bbox->priv;
 
   orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (widget));
-  spacing = gtk_box_get_spacing (box);
+  spacing = gtk_box_get_spacing (GTK_BOX (widget));
 
-  _gtk_button_box_child_requisition (widget,
-                                     &nvis_children,
-                                     NULL,
-                                     &child_width,
-                                     &child_height);
+  gtk_button_box_child_requisition (widget,
+                                    &nvis_children,
+                                    NULL,
+                                    &widths, &heights);
+
+  max_size = 0;
+  total_size = 0;
+  for (i = 0; i < nvis_children; i++)
+    {
+      if (orientation == GTK_ORIENTATION_HORIZONTAL)
+        {
+          total_size += widths[i];
+          max_size = MAX (max_size, heights[i]);
+        }
+      else
+        {
+          total_size += heights[i];
+          max_size = MAX (max_size, widths[i]);
+        }
+    }
+  g_free (widths);
+  g_free (heights);
 
   if (nvis_children == 0)
     {
@@ -512,11 +597,9 @@ gtk_button_box_size_request (GtkWidget      *widget,
         {
           case GTK_BUTTONBOX_SPREAD:
             if (orientation == GTK_ORIENTATION_HORIZONTAL)
-              requisition->width =
-                      nvis_children*child_width + ((nvis_children+1)*spacing);
+              requisition->width = total_size + ((nvis_children + 1)*spacing);
             else
-              requisition->height =
-                      nvis_children*child_height + ((nvis_children+1)*spacing);
+              requisition->height = total_size + ((nvis_children + 1)*spacing);
 
             break;
           case GTK_BUTTONBOX_EDGE:
@@ -524,11 +607,9 @@ gtk_button_box_size_request (GtkWidget      *widget,
           case GTK_BUTTONBOX_END:
           case GTK_BUTTONBOX_CENTER:
             if (orientation == GTK_ORIENTATION_HORIZONTAL)
-              requisition->width =
-                      nvis_children*child_width + ((nvis_children-1)*spacing);
+              requisition->width = total_size + ((nvis_children - 1)*spacing);
             else
-              requisition->height =
-                      nvis_children*child_height + ((nvis_children-1)*spacing);
+              requisition->height = total_size + ((nvis_children - 1)*spacing);
 
             break;
           default:
@@ -537,59 +618,132 @@ gtk_button_box_size_request (GtkWidget      *widget,
         }
 
       if (orientation == GTK_ORIENTATION_HORIZONTAL)
-        requisition->height = child_height;
+        requisition->height = max_size;
       else
-        requisition->width = child_width;
+        requisition->width = max_size;
     }
+}
 
-  border_width = gtk_container_get_border_width (GTK_CONTAINER (box));
-  requisition->width += border_width * 2;
-  requisition->height += border_width * 2;
+static void
+gtk_button_box_get_preferred_width (GtkWidget *widget,
+                                    gint      *minimum,
+                                    gint      *natural)
+{
+  GtkRequisition requisition;
+
+  gtk_button_box_size_request (widget, &requisition);
+
+  *minimum = *natural = requisition.width;
+}
+
+static void
+gtk_button_box_get_preferred_height (GtkWidget *widget,
+                                     gint      *minimum,
+                                     gint      *natural)
+{
+  GtkRequisition requisition;
+
+  gtk_button_box_size_request (widget, &requisition);
+
+  *minimum = *natural = requisition.height;
+}
+
+static void
+gtk_button_box_get_preferred_width_for_height (GtkWidget *widget,
+                                               gint       height,
+                                               gint      *minimum,
+                                               gint      *natural)
+{
+  gtk_button_box_get_preferred_width (widget, minimum, natural);
+}
+
+static void
+gtk_button_box_get_preferred_height_for_width (GtkWidget *widget,
+                                               gint       width,
+                                               gint      *minimum,
+                                               gint      *natural)
+{
+  gtk_button_box_get_preferred_height (widget, minimum, natural);
 }
 
 static void
 gtk_button_box_size_allocate (GtkWidget     *widget,
                               GtkAllocation *allocation)
 {
-  GtkButtonBoxPriv *priv;
-  GtkBox *base_box;
-  GtkButtonBox *box;
+  GtkButtonBoxPrivate *priv;
+  GtkButtonBox *bbox;
   GList *children, *list;
   GtkAllocation child_allocation;
   gint nvis_children;
+  gint n_primaries;
   gint n_secondaries;
-  gint child_width;
-  gint child_height;
   gint x = 0;
   gint y = 0;
   gint secondary_x = 0;
   gint secondary_y = 0;
   gint width = 0;
   gint height = 0;
-  gint childspace;
   gint childspacing = 0;
   gint spacing;
-  guint border_width;
   GtkOrientation orientation;
+  gint ipad_x, ipad_y;
+  gint *widths;
+  gint *heights;
+  gint *sizes;
+  gint primary_size;
+  gint secondary_size;
+  gint total_size;
+  gint i;
 
-  base_box = GTK_BOX (widget);
-  box = GTK_BUTTON_BOX (widget);
-  priv = box->priv;
+  bbox = GTK_BUTTON_BOX (widget);
+  priv = bbox->priv;
 
-  border_width = gtk_container_get_border_width (GTK_CONTAINER (box));
   orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (widget));
-  spacing = gtk_box_get_spacing (base_box);
-  _gtk_button_box_child_requisition (widget,
-                                     &nvis_children,
-                                     &n_secondaries,
-                                     &child_width,
-                                     &child_height);
-  widget->allocation = *allocation;
+  spacing = gtk_box_get_spacing (GTK_BOX (widget));
+
+  gtk_widget_style_get (widget,
+                        "child-internal-pad-x", &ipad_x,
+                        "child-internal-pad-y", &ipad_y,
+                        NULL);
+  gtk_button_box_child_requisition (widget,
+                                    &nvis_children,
+                                    &n_secondaries,
+                                    &widths, &heights);
+
+  n_primaries = nvis_children - n_secondaries;
+  primary_size = 0;
+  secondary_size = 0;
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    sizes = widths;
+  else
+    sizes = heights;
+
+  i = 0;
+  list = children = _gtk_box_get_children (GTK_BOX (widget));
+  while (children)
+    {
+      GtkWidget *child;
+
+      child = children->data;
+      children = children->next;
+
+      if (gtk_widget_get_visible (child))
+        {
+          if (gtk_button_box_get_child_secondary (bbox, child))
+            secondary_size += sizes[i];
+          else
+            primary_size += sizes[i];
+          i++;
+        }
+    }
+  total_size = primary_size + secondary_size;
+
+  gtk_widget_set_allocation (widget, allocation);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    width = allocation->width - border_width*2;
+    width = allocation->width;
   else
-    height = allocation->height - border_width*2;
+    height = allocation->height;
 
   switch (priv->layout_style)
     {
@@ -597,19 +751,15 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
 
         if (orientation == GTK_ORIENTATION_HORIZONTAL)
           {
-            childspacing = (width - (nvis_children * child_width))
-                    / (nvis_children + 1);
-            x = allocation->x + border_width + childspacing;
-            secondary_x = x + ((nvis_children - n_secondaries)
-                            * (child_width + childspacing));
+            childspacing = (width - total_size) / (nvis_children + 1);
+            x = allocation->x + childspacing;
+            secondary_x = x + primary_size + n_primaries * childspacing;
           }
         else
           {
-            childspacing = (height - (nvis_children * child_height))
-                    / (nvis_children + 1);
-            y = allocation->y + border_width + childspacing;
-            secondary_y = y + ((nvis_children - n_secondaries)
-                            * (child_height + childspacing));
+            childspacing = (height - total_size) / (nvis_children + 1);
+            y = allocation->y + childspacing;
+            secondary_y = y + primary_size + n_primaries * childspacing;
           }
 
         break;
@@ -620,36 +770,44 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
           {
             if (nvis_children >= 2)
               {
-                childspacing = (width - (nvis_children * child_width))
-                      / (nvis_children - 1);
-                x = allocation->x + border_width;
-                secondary_x = x + ((nvis_children - n_secondaries)
-                                   * (child_width + childspacing));
+                childspacing = (width - total_size) / (nvis_children - 1);
+                x = allocation->x;
+                secondary_x = x + primary_size + n_primaries * childspacing;
+              }
+            else if (nvis_children == 1)
+              {
+                /* one child, just center */
+                childspacing = width;
+                x = secondary_x = allocation->x
+                                  + (allocation->width - widths[0]) / 2;
               }
             else
               {
-                /* one or zero children, just center */
+                /* zero children, meh */
                 childspacing = width;
-                x = secondary_x = allocation->x
-                      + (allocation->width - child_width) / 2;
+                x = secondary_x = allocation->x + allocation->width / 2;
               }
           }
         else
           {
             if (nvis_children >= 2)
               {
-                childspacing = (height - (nvis_children*child_height))
-                        / (nvis_children-1);
-                y = allocation->y + border_width;
-                secondary_y = y + ((nvis_children - n_secondaries)
-                                * (child_height + childspacing));
+                childspacing = (height - total_size) / (nvis_children - 1);
+                y = allocation->y;
+                secondary_y = y + primary_size + n_primaries * childspacing;
+              }
+            else if (nvis_children == 1)
+              {
+                /* one child, just center */
+                childspacing = height;
+                y = secondary_y = allocation->y
+                                     + (allocation->height - heights[0]) / 2;
               }
             else
               {
-                /* one or zero children, just center */
+                /* zero children, meh */
                 childspacing = height;
-                y = secondary_y = allocation->y
-                        + (allocation->height - child_height) / 2;
+                y = secondary_y = allocation->y + allocation->height / 2;
               }
           }
 
@@ -660,20 +818,16 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
         if (orientation == GTK_ORIENTATION_HORIZONTAL)
           {
             childspacing = spacing;
-            x = allocation->x + border_width;
+            x = allocation->x;
             secondary_x = allocation->x + allocation->width
-              - child_width * n_secondaries
-              - spacing * (n_secondaries - 1)
-              - border_width;
+              - secondary_size - spacing * (n_secondaries - 1);
           }
         else
           {
             childspacing = spacing;
-            y = allocation->y + border_width;
+            y = allocation->y;
             secondary_y = allocation->y + allocation->height
-              - child_height * n_secondaries
-              - spacing * (n_secondaries - 1)
-              - border_width;
+              - secondary_size - spacing * (n_secondaries - 1);
           }
 
         break;
@@ -684,19 +838,15 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
           {
             childspacing = spacing;
             x = allocation->x + allocation->width
-              - child_width * (nvis_children - n_secondaries)
-              - spacing * (nvis_children - n_secondaries - 1)
-              - border_width;
-            secondary_x = allocation->x + border_width;
+              - primary_size - spacing * (n_primaries - 1);
+            secondary_x = allocation->x;
           }
         else
           {
             childspacing = spacing;
             y = allocation->y + allocation->height
-              - child_height * (nvis_children - n_secondaries)
-              - spacing * (nvis_children - n_secondaries - 1)
-              - border_width;
-            secondary_y = allocation->y + border_width;
+              - primary_size - spacing * (n_primaries - 1);
+            secondary_y = allocation->y;
           }
 
         break;
@@ -708,20 +858,18 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
             childspacing = spacing;
             x = allocation->x +
               (allocation->width
-               - (child_width * (nvis_children - n_secondaries)
-               + spacing * (nvis_children - n_secondaries - 1))) / 2
-              + (n_secondaries * child_width + n_secondaries * spacing) / 2;
-            secondary_x = allocation->x + border_width;
+               - (primary_size + spacing * (n_primaries - 1))) / 2
+              + (secondary_size + n_secondaries * spacing) / 2;
+            secondary_x = allocation->x;
           }
         else
           {
             childspacing = spacing;
             y = allocation->y +
               (allocation->height
-               - (child_height * (nvis_children - n_secondaries)
-                  + spacing * (nvis_children - n_secondaries - 1))) / 2
-              + (n_secondaries * child_height + n_secondaries * spacing) / 2;
-            secondary_y = allocation->y + border_width;
+               - (primary_size + spacing * (n_primaries - 1))) / 2
+              + (secondary_size + n_secondaries * spacing) / 2;
+            secondary_y = allocation->y;
           }
 
         break;
@@ -731,74 +879,63 @@ gtk_button_box_size_allocate (GtkWidget     *widget,
         break;
     }
 
-    if (orientation == GTK_ORIENTATION_HORIZONTAL)
-      {
-        y = allocation->y + (allocation->height - child_height) / 2;
-        childspace = child_width + childspacing;
-      }
-    else
-      {
-        x = allocation->x + (allocation->width - child_width) / 2;
-        childspace = child_height + childspacing;
-      }
-
-  list = children = _gtk_box_get_children (GTK_BOX (box));
-
+  children = list;
+  i = 0;
   while (children)
     {
       GtkWidget *child;
-      gboolean is_secondary;
 
       child = children->data;
       children = children->next;
 
-      is_secondary = gtk_button_box_get_child_secondary (box, child);
-
       if (gtk_widget_get_visible (child))
         {
-          child_allocation.width = child_width;
-          child_allocation.height = child_height;
+          child_allocation.width = widths[i];
+          child_allocation.height = heights[i];
 
           if (orientation == GTK_ORIENTATION_HORIZONTAL)
             {
-              child_allocation.y = y;
+              child_allocation.y = allocation->y + (allocation->height - child_allocation.height) / 2;
 
-              if (is_secondary)
+              if (gtk_button_box_get_child_secondary (bbox, child))
                 {
                   child_allocation.x = secondary_x;
-                  secondary_x += childspace;
+                  secondary_x += child_allocation.width + childspacing;
                 }
               else
                 {
                   child_allocation.x = x;
-                  x += childspace;
+                  x += child_allocation.width + childspacing;
                 }
 
               if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
                   child_allocation.x = (allocation->x + allocation->width)
-                          - (child_allocation.x + child_width - allocation->x);
+                          - (child_allocation.x + child_allocation.width - allocation->x);
             }
           else
             {
-              child_allocation.x = x;
+              child_allocation.x = allocation->x + (allocation->width - child_allocation.width) / 2;
 
-              if (is_secondary)
+              if (gtk_button_box_get_child_secondary (bbox, child))
                 {
                   child_allocation.y = secondary_y;
-                  secondary_y += childspace;
+                  secondary_y += child_allocation.height + childspacing;
                 }
               else
                 {
                   child_allocation.y = y;
-                  y += childspace;
+                  y += child_allocation.height + childspacing;
                 }
             }
 
           gtk_widget_size_allocate (child, &child_allocation);
+          i++;
         }
     }
 
   g_list_free (list);
+  g_free (widths);
+  g_free (heights);
 }
 
 /**
