@@ -1736,18 +1736,15 @@ get_size (GtkCellRenderer    *cell,
 
   pango_layout_get_pixel_extents (layout, NULL, &rect);
 
-  if (height)
-    *height = ypad * 2 + rect.height;
-
-  if (width)
-    *width = xpad * 2 + rect.x + rect.width;
-
   if (cell_area)
     {
       gfloat xalign, yalign;
 
       gtk_cell_renderer_get_alignment (cell, &xalign, &yalign);
 
+      rect.height = MIN (rect.height, cell_area->height - 2 * ypad);
+      rect.width  = MIN (rect.width, cell_area->width - (2 * xpad) - rect.x);
+
       if (x_offset)
 	{
 	  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
@@ -1770,6 +1767,12 @@ get_size (GtkCellRenderer    *cell,
       if (y_offset) *y_offset = 0;
     }
 
+  if (height)
+    *height = ypad * 2 + rect.height;
+
+  if (width)
+    *width = xpad * 2 + rect.x + rect.width;
+
   g_object_unref (layout);
 }
 
@@ -2157,19 +2160,19 @@ gtk_cell_renderer_text_get_preferred_width (GtkCellRenderer *cell,
   if ((priv->ellipsize_set && priv->ellipsize != PANGO_ELLIPSIZE_NONE) || priv->width_chars > 0)
     min_width = 
       xpad * 2 + 
-      MIN (PANGO_PIXELS (text_width), 
+      MIN (PANGO_PIXELS_CEIL (text_width), 
 	   (PANGO_PIXELS (char_width) * MAX (priv->width_chars, ellipsize_chars)));
   /* If no width-chars set, minimum for wrapping text will be the wrap-width */
   else if (priv->wrap_width > -1)
-    min_width = xpad * 2 + rect.x + MIN (PANGO_PIXELS (text_width), priv->wrap_width);
+    min_width = xpad * 2 + rect.x + MIN (PANGO_PIXELS_CEIL (text_width), priv->wrap_width);
   else
-    min_width = xpad * 2 + rect.x + PANGO_PIXELS (text_width);
+    min_width = xpad * 2 + rect.x + PANGO_PIXELS_CEIL (text_width);
 
   if (priv->width_chars > 0)
     nat_width = xpad * 2 + 
-      MAX ((PANGO_PIXELS (char_width) * priv->width_chars), PANGO_PIXELS (text_width));
+      MAX ((PANGO_PIXELS (char_width) * priv->width_chars), PANGO_PIXELS_CEIL (text_width));
   else
-    nat_width = xpad * 2 + PANGO_PIXELS (text_width);
+    nat_width = xpad * 2 + PANGO_PIXELS_CEIL (text_width);
 
 
   nat_width = MAX (nat_width, min_width);
