@@ -26,7 +26,8 @@
 
 #include "config.h"
 
-#include "gtkmenu.h"
+#include "gtkmenuprivate.h"
+#include "gtkmenuitemprivate.h"
 #include "gtktearoffmenuitem.h"
 #include "gtkintl.h"
 
@@ -39,13 +40,17 @@ struct _GtkTearoffMenuItemPrivate
   guint torn_off : 1;
 };
 
-static void gtk_tearoff_menu_item_size_request (GtkWidget             *widget,
-				                GtkRequisition        *requisition);
-static gint gtk_tearoff_menu_item_expose     (GtkWidget             *widget,
-					      GdkEventExpose        *event);
-static void gtk_tearoff_menu_item_activate   (GtkMenuItem           *menu_item);
-static void gtk_tearoff_menu_item_parent_set (GtkWidget             *widget,
-					      GtkWidget             *previous);
+static void gtk_tearoff_menu_item_get_preferred_width  (GtkWidget      *widget,
+							gint           *minimum,
+							gint           *natural);
+static void gtk_tearoff_menu_item_get_preferred_height (GtkWidget      *widget,
+							gint           *minimum,
+							gint           *natural);
+static gboolean gtk_tearoff_menu_item_draw             (GtkWidget      *widget,
+							cairo_t        *cr);
+static void gtk_tearoff_menu_item_activate             (GtkMenuItem    *menu_item);
+static void gtk_tearoff_menu_item_parent_set           (GtkWidget      *widget,
+							GtkWidget      *previous);
 
 G_DEFINE_TYPE (GtkTearoffMenuItem, gtk_tearoff_menu_item, GTK_TYPE_MENU_ITEM)
 
@@ -64,8 +69,9 @@ gtk_tearoff_menu_item_class_init (GtkTearoffMenuItemClass *klass)
   widget_class = (GtkWidgetClass*) klass;
   menu_item_class = (GtkMenuItemClass*) klass;
 
-  widget_class->expose_event = gtk_tearoff_menu_item_expose;
-  widget_class->size_request = gtk_tearoff_menu_item_size_request;
+  widget_class->draw = gtk_tearoff_menu_item_draw;
+  widget_class->get_preferred_width = gtk_tearoff_menu_item_get_preferred_width;
+  widget_class->get_preferred_height = gtk_tearoff_menu_item_get_preferred_height;
   widget_class->parent_set = gtk_tearoff_menu_item_parent_set;
 
   menu_item_class->activate = gtk_tearoff_menu_item_activate;
@@ -87,8 +93,23 @@ gtk_tearoff_menu_item_init (GtkTearoffMenuItem *tearoff_menu_item)
 }
 
 static void
-gtk_tearoff_menu_item_size_request (GtkWidget      *widget,
-				    GtkRequisition *requisition)
+gtk_tearoff_menu_item_get_preferred_width (GtkWidget      *widget,
+					   gint           *minimum,
+					   gint           *natural)
+{
+  GtkStyle *style;
+  guint border_width;
+
+  style = gtk_widget_get_style (widget);
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+  *minimum = *natural = (border_width + style->xthickness + BORDER_SPACING) * 2;
+}
+
+static void
+gtk_tearoff_menu_item_get_preferred_height (GtkWidget      *widget,
+					    gint           *minimum,
+					    gint           *natural)
 {
   GtkStyle *style;
   GtkWidget *parent;
@@ -97,31 +118,30 @@ gtk_tearoff_menu_item_size_request (GtkWidget      *widget,
   style = gtk_widget_get_style (widget);
 
   border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-  requisition->width = (border_width + style->xthickness + BORDER_SPACING) * 2;
-  requisition->height = (border_width + style->ythickness) * 2;
+  *minimum = *natural = (border_width + style->ythickness) * 2;
 
   parent = gtk_widget_get_parent (widget);
-  if (GTK_IS_MENU (parent) && GTK_MENU (parent)->torn_off)
+  if (GTK_IS_MENU (parent) && GTK_MENU (parent)->priv->torn_off)
     {
-      requisition->height += ARROW_SIZE;
+      *minimum += ARROW_SIZE;
+      *natural += ARROW_SIZE;
     }
   else
     {
-      requisition->height += style->ythickness + 4;
+      *minimum += style->ythickness + 4;
+      *natural += style->ythickness + 4;
     }
 }
 
-static void
-gtk_tearoff_menu_item_paint (GtkWidget   *widget,
-			     GdkRectangle *area)
+static gboolean
+gtk_tearoff_menu_item_draw (GtkWidget *widget,
+                            cairo_t   *cr)
 {
-  GtkAllocation allocation;
   GtkMenuItem *menu_item;
   GtkShadowType shadow_type;
   GtkStateType state;
   GtkStyle *style;
-  gint width, height;
-  gint x, y;
+  gint x, y, width, height;
   gint right_max;
   guint border_width;
   GtkArrowType arrow_type;
@@ -129,111 +149,98 @@ gtk_tearoff_menu_item_paint (GtkWidget   *widget,
   GtkWidget *parent;
   GdkWindow *window;
 
-  if (gtk_widget_is_drawable (widget))
+  menu_item = GTK_MENU_ITEM (widget);
+  style = gtk_widget_get_style (widget);
+  window = gtk_widget_get_window (widget);
+  direction = gtk_widget_get_direction (widget);
+  state = gtk_widget_get_state (widget);
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (menu_item));
+  x = border_width;
+  y = border_width;
+  width = gtk_widget_get_allocated_width (widget) - border_width * 2;
+  height = gtk_widget_get_allocated_height (widget) - border_width * 2;
+  right_max = x + width;
+
+  if (state == GTK_STATE_PRELIGHT)
     {
-      menu_item = GTK_MENU_ITEM (widget);
+      gint selected_shadow_type;
+      
+      gtk_widget_style_get (widget,
+                            "selected-shadow-type", &selected_shadow_type,
+                            NULL);
+      gtk_paint_box (style,
+                     cr,
+                     GTK_STATE_PRELIGHT,
+                     selected_shadow_type,
+                     widget, "menuitem",
+                     x, y, width, height);
+    }
 
-      style = gtk_widget_get_style (widget);
-      window = gtk_widget_get_window (widget);
-      direction = gtk_widget_get_direction (widget);
-
-      border_width = gtk_container_get_border_width (GTK_CONTAINER (menu_item));
-      gtk_widget_get_allocation (widget, &allocation);
-      x = allocation.x + border_width;
-      y = allocation.y + border_width;
-      width = allocation.width - border_width * 2;
-      height = allocation.height - border_width * 2;
-      right_max = x + width;
-
-      state = gtk_widget_get_state (widget);
+  parent = gtk_widget_get_parent (widget);
+  if (GTK_IS_MENU (parent) && GTK_MENU (parent)->priv->torn_off)
+    {
+      gint arrow_x;
 
       if (state == GTK_STATE_PRELIGHT)
-	{
-	  gint selected_shadow_type;
-	  
-	  gtk_widget_style_get (widget,
-				"selected-shadow-type", &selected_shadow_type,
-				NULL);
-          gtk_paint_box (style,
-                         window,
-			 GTK_STATE_PRELIGHT,
-			 selected_shadow_type,
-			 area, widget, "menuitem",
-			 x, y, width, height);
-	}
+        shadow_type = GTK_SHADOW_IN;
       else
-	gdk_window_clear_area (window, area->x, area->y, area->width, area->height);
+        shadow_type = GTK_SHADOW_OUT;
 
-      parent = gtk_widget_get_parent (widget);
-      if (GTK_IS_MENU (parent) && GTK_MENU (parent)->torn_off)
-	{
-	  gint arrow_x;
-
-          if (state == GTK_STATE_PRELIGHT)
-	    shadow_type = GTK_SHADOW_IN;
-	  else
-	    shadow_type = GTK_SHADOW_OUT;
-
-	  if (menu_item->toggle_size > ARROW_SIZE)
-	    {
-	      if (direction == GTK_TEXT_DIR_LTR) {
-		arrow_x = x + (menu_item->toggle_size - ARROW_SIZE)/2;
-		arrow_type = GTK_ARROW_LEFT;
-	      }
-	      else {
-		arrow_x = x + width - menu_item->toggle_size + (menu_item->toggle_size - ARROW_SIZE)/2; 
-		arrow_type = GTK_ARROW_RIGHT;	    
-	      }
-	      x += menu_item->toggle_size + BORDER_SPACING;
-	    }
-	  else
-	    {
-	      if (direction == GTK_TEXT_DIR_LTR) {
-		arrow_x = ARROW_SIZE / 2;
-		arrow_type = GTK_ARROW_LEFT;
-	      }
-	      else {
-		arrow_x = x + width - 2 * ARROW_SIZE + ARROW_SIZE / 2; 
-		arrow_type = GTK_ARROW_RIGHT;	    
-	      }
-	      x += 2 * ARROW_SIZE;
-	    }
+      if (menu_item->priv->toggle_size > ARROW_SIZE)
+        {
+          if (direction == GTK_TEXT_DIR_LTR) {
+            arrow_x = x + (menu_item->priv->toggle_size - ARROW_SIZE)/2;
+            arrow_type = GTK_ARROW_LEFT;
+          }
+          else {
+            arrow_x = x + width - menu_item->priv->toggle_size + (menu_item->priv->toggle_size - ARROW_SIZE)/2;
+            arrow_type = GTK_ARROW_RIGHT;
+          }
+          x += menu_item->priv->toggle_size + BORDER_SPACING;
+        }
+      else
+        {
+          if (direction == GTK_TEXT_DIR_LTR)
+            {
+              arrow_x = ARROW_SIZE / 2;
+              arrow_type = GTK_ARROW_LEFT;
+            }
+          else
+            {
+              arrow_x = x + width - 2 * ARROW_SIZE + ARROW_SIZE / 2;
+              arrow_type = GTK_ARROW_RIGHT;
+            }
+          x += 2 * ARROW_SIZE;
+        }
 
 
-          gtk_paint_arrow (style, window,
-                           state, shadow_type,
-			   NULL, widget, "tearoffmenuitem",
-			   arrow_type, FALSE,
-			   arrow_x, y + height / 2 - 5, 
-			   ARROW_SIZE, ARROW_SIZE);
-	}
-
-      while (x < right_max)
-	{
-	  gint x1, x2;
-
-	  if (direction == GTK_TEXT_DIR_LTR) {
-	    x1 = x;
-	    x2 = MIN (x + TEAR_LENGTH, right_max);
-	  }
-	  else {
-	    x1 = right_max - x;
-	    x2 = MAX (right_max - x - TEAR_LENGTH, 0);
-	  }
-
-          gtk_paint_hline (style, window, GTK_STATE_NORMAL,
-			   NULL, widget, "tearoffmenuitem",
-                           x1, x2, y + (height - style->ythickness) / 2);
-	  x += 2 * TEAR_LENGTH;
-	}
+      gtk_paint_arrow (style, cr,
+                       state, shadow_type,
+                       widget, "tearoffmenuitem",
+                       arrow_type, FALSE,
+                       arrow_x, y + height / 2 - 5,
+                       ARROW_SIZE, ARROW_SIZE);
     }
-}
 
-static gint
-gtk_tearoff_menu_item_expose (GtkWidget      *widget,
-			    GdkEventExpose *event)
-{
-  gtk_tearoff_menu_item_paint (widget, &event->area);
+  while (x < right_max)
+    {
+      gint x1, x2;
+
+      if (direction == GTK_TEXT_DIR_LTR) {
+        x1 = x;
+        x2 = MIN (x + TEAR_LENGTH, right_max);
+      }
+      else {
+        x1 = right_max - x;
+        x2 = MAX (right_max - x - TEAR_LENGTH, 0);
+      }
+
+      gtk_paint_hline (style, cr, GTK_STATE_NORMAL,
+                       widget, "tearoffmenuitem",
+                       x1, x2, y + (height - style->ythickness) / 2);
+      x += 2 * TEAR_LENGTH;
+    }
 
   return FALSE;
 }
@@ -250,7 +257,7 @@ gtk_tearoff_menu_item_activate (GtkMenuItem *menu_item)
 
       gtk_widget_queue_resize (GTK_WIDGET (menu_item));
       gtk_menu_set_tearoff_state (GTK_MENU (parent),
-				  !menu->torn_off);
+				  !menu->priv->torn_off);
     }
 }
 
